import sys
#import CSHMM_TF_analysis_release as ANA
import CSHMM_TF_train_release as ML
import numpy as np
import json
import pdb

def json_visualization( gene_names,cell_names,cell_exps,model=None, hid_var=None, model_file= None):
    if model is None or hid_var is None:
        model,hid_var=ML.load_model(model_file)
    path_info=model["path_info"]
    g_param=model["g_param"]
    sigma_param=model['sigma_param']
    K_param=model['K_param']
    gene_start_time=model['gene_start_time']
    cell_path = hid_var["cell_path"]
    cell_time = hid_var["cell_time"]
    edges=[]
    node_dict={}
    tfs=[]
    tf_count=0
    ML.dTD=ML.getdTD("Mouse_TF_targets.txt")
    TF_names = np.array(map(lambda x: x.lower(),ML.dTD.keys()))
    #pdb.set_trace()
    for i,path in enumerate(path_info):
        edges+=[{"id":path["ID"], "source_node":path["Sp_idx"], "end_node":path["Sc_idx"]}]
        node_dict[path["Sp_idx"]]={"id": path["Sp_idx"], "time": path["level"], "expression":g_param[path["Sp_idx"],:].tolist()}
        node_dict[path["Sc_idx"]]={"id": path["Sc_idx"], "time": path["level"]+1, "expression":g_param[path["Sc_idx"],:].tolist()}
        if "TF_list" in path.keys():
            for tf,tfp in zip(path["TF_list"],path["TF_pvalue"]):
				if tf in TF_names.tolist():
					tf_idx=TF_names.tolist().index(tf)
					tfs+=[{ "id":tf_count, "tf_name": tf, "location": (path["ID"], model["TF_start_time"][path["ID"],tf_idx]), "p-value":tfp}]
					tf_count+=1
    nodes=node_dict.values()
    cells=[]
    for i,(cn,ce,cl,cp,ct) in enumerate(zip(cell_names,cell_exps,hid_var["cell_labels"],hid_var["cell_path"],hid_var["cell_time"])):
        p=cell_path[i]
        t=cell_time[i]
        Sp_idx=path_info[p]['Sp_idx']
        Sc_idx=path_info[p]['Sc_idx']
        g_a=g_param[Sp_idx]
        g_b=g_param[Sc_idx]
        mu_x_i=g_b+(g_a-g_b)*np.exp(-K_param[p]*np.maximum(t-gene_start_time[p],0))
        diff = cell_exps[i]-mu_x_i
        ssq = np.sum(diff**2)
        sign = np.sign(np.sum(diff))
        cells+=[{ "id":cn,"type":cl, "location": (cp , ct, sign*ssq/len(gene_names))}]
    genes=gene_names.tolist()
    output=open(model_file+".json","w")
    output.write(json.dumps([nodes,edges,cells,genes,tfs], sort_keys=True,indent=4))
    output.close()
if __name__=="__main__":
    if len(sys.argv)<3:
        print ("usage: python CSHMM_json.py data_file_name model_file_name [-ng number_of_genes_used_for_traning] ")
        sys.exit()
    data_file=sys.argv[1]
    model_file = sys.argv[2]
    ng = 100000
    if len(sys.argv)>3 and sys.argv[3]=='-ng':
        ng = int(sys.argv[4])
    cell_names,cell_day,cell_labels,cell_exps,gene_names=ML.load_data(data_file,ng)
    json_visualization(gene_names,cell_names,cell_exps, model_file= model_file)


